@@ -9,8 +9,6 @@ use Plack::Request;
 use Data::Transform::ExplicitMetadata;
 use URI::Escape qw(uri_escape);
 
-our $VERSION = '0.25';
-
 sub response_url_base() { '/watchpoints' };
 
 __PACKAGE__->add_route('put', qr{/watchpoints/(.+)}, 'set');
@@ -106,7 +104,7 @@ sub Devel::hdb::App::notify_watch_expr {
 
 =head1 NAME
 
-Devel::hdb::App::WatchPoint - Get and set watchpoints
+Devel::hdb::App::Watchpoint - Get and set watchpoints
 
 =head1 DESCRIPTION
 
@@ -158,7 +156,7 @@ Returns 404 if there is no watchpoint with that expr.
 
 =head1 SEE ALSO
 
-L<Devel::hdb>
+Devel::hdb
 
 =head1 AUTHOR
 
@@ -166,5 +164,5 @@ Anthony Brummett <brummett@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright 2018, Anthony Brummett.  This module is free software. It may
+Copyright 2014, Anthony Brummett.  This module is free software. It may
 be used, redistributed and/or modified under the same terms as Perl itself.
